@@ -1,5 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
+import weakref
+
 from qtpy.QtCore import Qt
 from qtpy import QtCore, QtGui, QtWidgets
 from glue import core
@@ -21,7 +23,7 @@ class GlueMdiArea(QtWidgets.QMdiArea):
         :type application: :class:`~glue.app.qt.application.GlueApplication`
         """
         super(GlueMdiArea, self).__init__(parent)
-        self._application = application
+        self._application = weakref.ref(application)
         self.setAcceptDrops(True)
         self.setAttribute(Qt.WA_DeleteOnClose)
         self.setBackground(QtGui.QBrush(QtGui.QColor(250, 250, 250)))
@@ -47,11 +49,14 @@ class GlueMdiArea(QtWidgets.QMdiArea):
         md = event.mimeData()
 
         def new_layer(layer):
+            application = self._application()
+            if application is None:
+                return
             if isinstance(layer, core.data.Data):
-                self._application.choose_new_data_viewer(layer)
+                application.choose_new_data_viewer(layer)
             else:
                 assert isinstance(layer, core.subset.Subset)
-                self._application.choose_new_data_viewer(layer.data)
+                application.choose_new_data_viewer(layer.data)
 
         if md.hasFormat(LAYER_MIME_TYPE):
             new_layer(md.data(LAYER_MIME_TYPE))
