@@ -6,7 +6,6 @@ import os
 import sys
 
 import pytest
-from qtpy import QtWidgets
 import numpy as np
 from mock import patch, MagicMock
 
@@ -15,10 +14,11 @@ from glue.core.data import Data
 from glue.core.component_link import ComponentLink
 from glue.core.data_collection import DataCollection
 from glue.core.tests.test_state import Cloner, containers_equal, doubler, clone
-from glue.tests.helpers import requires_ipython
+from glue.tests.helpers import requires_ipython, PYSIDE2_INSTALLED  # noqa
 from glue.viewers.image.qt import ImageViewer
 from glue.viewers.scatter.qt import ScatterViewer
 from glue.viewers.histogram.qt import HistogramViewer
+from glue.utils.qt import get_qapp
 
 
 from ..application import GlueApplication, GlueLogger
@@ -46,20 +46,24 @@ class TestGlueApplication(object):
         self.app.new_tab()
         assert tab_count(self.app) == t0 + 1
 
-    def test_save_session(self):
-        self.app.save_session = MagicMock()
-        with patch('qtpy.compat.getsavefilename') as fd:
-            fd.return_value = '/tmp/junk', 'jnk'
-            self.app._choose_save_session()
-            self.app.save_session.assert_called_once_with('/tmp/junk.glu', include_data=False)
+    def test_save_session_ok(self):
+        with patch.object(self.app, 'save_session') as save:
+            with patch('qtpy.compat.getsavefilename') as fd:
+                fd.return_value = '/tmp/junk', 'jnk'
+                self.app._choose_save_session()
+                save.assert_called_once_with('/tmp/junk.glu', include_data=False, absolute_paths=False)
+                fd.reset_mock()
+            save.reset_mock()
 
     def test_save_session_cancel(self):
         """shouldnt try to save file if no file name provided"""
-        self.app.save_session = MagicMock()
-        with patch('qtpy.compat.getsavefilename') as fd:
-            fd.return_value = '', 'jnk'
-            self.app._choose_save_session()
-            assert self.app.save_session.call_count == 0
+        with patch.object(self.app, 'save_session') as save:
+            with patch('glue.app.qt.application.compat.getsavefilename') as fd:
+                fd.return_value = '', 'jnk'
+                self.app._choose_save_session()
+                assert save.call_count == 0
+                fd.reset_mock()
+            save.reset_mock()
 
     def test_choose_save_session_ioerror(self):
         """should show box on ioerror"""
@@ -74,6 +78,9 @@ class TestGlueApplication(object):
                 with patch('qtpy.QtWidgets.QMessageBox') as mb:
                     self.app._choose_save_session()
                     assert mb.call_count == 1
+                    mb.reset_mock()
+                op.reset_mock()
+            fd.reset_mock()
 
     @requires_ipython
     def test_terminal_present(self):
@@ -87,16 +94,19 @@ class TestGlueApplication(object):
 
     @requires_ipython
     def test_toggle_terminal(self):
-        term = MagicMock()
-        self.app._terminal = term
+        with patch.object(self.app, '_terminal') as term:
 
-        term.isVisible.return_value = False
-        self.app._button_ipython.click()
-        assert term.show.call_count == 1
+            self.app._terminal = term
 
-        term.isVisible.return_value = True
-        self.app._button_ipython.click()
-        assert term.hide.call_count == 1
+            term.isVisible.return_value = False
+            self.app._button_ipython.click()
+            assert term.show.call_count == 1
+
+            term.isVisible.return_value = True
+            self.app._button_ipython.click()
+            assert term.hide.call_count == 1
+
+            term.reset_mock()
 
     def test_close_tab(self):
 
@@ -130,7 +140,10 @@ class TestGlueApplication(object):
             self.app.choose_new_data_viewer()
             assert len(self.app.current_tab.subWindowList()) == ct
 
-    def test_new_data_viewer(self):
+            pc.reset_mock()
+
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
+    def test_new_data_viewer_ok(self):
 
         with patch('glue.app.qt.application.pick_class') as pc:
 
@@ -141,13 +154,19 @@ class TestGlueApplication(object):
             viewer = self.app.choose_new_data_viewer()
             assert len(self.app.current_tab.subWindowList()) == ct + 1
             viewer.close()
+            # TODO: figure out why this doesn't work as expected
+            # assert len(self.app.current_tab.subWindowList()) == ct
 
+            pc.reset_mock()
+
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     def test_move(self):
         viewer = self.app.new_data_viewer(ScatterViewer)
         viewer.move(10, 20)
         assert viewer.position == (10, 20)
         viewer.close()
 
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     def test_resize(self):
         viewer = self.app.new_data_viewer(ScatterViewer)
         viewer.viewer_size = (100, 200)
@@ -170,6 +189,8 @@ class TestGlueApplication(object):
             args, kwargs = pc.call_args
             assert kwargs['default'] is ImageViewer
 
+            pc.reset_mock()
+
     def test_drop_load_data(self):
 
         load_data = MagicMock()
@@ -214,6 +235,9 @@ class TestGlueApplication(object):
             self.app.dropEvent(e)
             assert mb.call_count == 1
             assert "When dragging and dropping files" in mb.call_args[0][2]
+            mb.reset_mock()
+
+        load_data.reset_mock()
 
     def test_subset_facet(self):
         # regression test for 335
@@ -312,7 +336,7 @@ class TestApplicationSession(object):
 
     def test_data_application(self):
         dc = DataCollection([Data(label='test',
-                                            x=[1, 2, 3], y=[2, 3, 4])])
+                                  x=[1, 2, 3], y=[2, 3, 4])])
         app = GlueApplication(dc)
         self.check_clone(app)
 
@@ -328,6 +352,7 @@ class TestApplicationSession(object):
         app = GlueApplication(dc)
         self.check_clone(app)
 
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     def test_scatter_viewer(self):
         d = Data(label='x', x=[1, 2, 3, 4, 5], y=[2, 3, 4, 5, 6])
         dc = DataCollection([d])
@@ -349,6 +374,7 @@ class TestApplicationSession(object):
         copy1.close()
         copy2.close()
 
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     def test_multi_tab(self):
         d = Data(label='hist', x=[[1, 2], [2, 3]])
         dc = DataCollection([d])
@@ -364,6 +390,7 @@ class TestApplicationSession(object):
         app.close()
         copy.close()
 
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     def test_histogram(self):
         d = Data(label='hist', x=[[1, 2], [2, 3]])
         dc = DataCollection([d])
@@ -398,6 +425,45 @@ class TestApplicationSession(object):
         sg.style.color = '#112233'
         assert sg.subsets[0].style.color == '#112233'
 
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
+    def test_deselect_tool_on_viewer_change(self):
+
+        d = Data(label='hist', x=[[1, 2], [2, 3]])
+        dc = DataCollection([d])
+
+        app = GlueApplication(dc)
+        v1 = app.new_data_viewer(HistogramViewer, data=d)
+        v2 = app.new_data_viewer(HistogramViewer, data=d)
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool is None
+
+        v2.toolbar.active_tool = 'select:xrange'
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool.tool_id == 'select:xrange'
+
+        app.current_tab.activateNextSubWindow()
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool is None
+
+        v1.toolbar.active_tool = 'select:xrange'
+
+        # Emit a signal without changing the active subWindow to make sure that
+        # the tool doesn't get reset.
+        app.current_tab.subWindowActivated.emit(app.current_tab.activeSubWindow())
+
+        assert v1.toolbar.active_tool.tool_id == 'select:xrange'
+        assert v2.toolbar.active_tool is None
+
+        app.current_tab.activateNextSubWindow()
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool is None
+
+        app.close()
+
 
 def test_logger_close():
 
@@ -406,7 +472,9 @@ def test_logger_close():
 
     app = GlueApplication()
     app.close()
-    app.app.processEvents()
+
+    qapp = get_qapp()
+    qapp.processEvents()
 
     assert not isinstance(sys.stderr, GlueLogger)
 
