@@ -1,7 +1,6 @@
 from __future__ import absolute_import, division, print_function
 
 import logging
-import operator
 import warnings
 
 import numpy as np
@@ -68,11 +67,6 @@ class Component(object):
         else:
             self._units = str(value)
 
-    @property
-    def hidden(self):
-        """Whether the Component is hidden by default"""
-        return False
-
     @property
     def data(self):
         """ The underlying :class:`numpy.ndarray` """
@@ -110,6 +104,13 @@ class Component(object):
         """
         return False
 
+    @property
+    def datetime(self):
+        """
+        Whether or not or not the datatype is a date/time
+        """
+        return False
+
     def __str__(self):
         return "%s with shape %s" % (self.__class__.__name__, shape_to_string(self.shape))
 
@@ -205,6 +206,9 @@ class Component(object):
         if np.issubdtype(data.dtype, np.object_):
             return CategoricalComponent(data, units=units)
 
+        if data.dtype.kind == 'M':
+            return DateTimeComponent(data)
+
         n = coerce_numeric(data)
 
         thresh = 0.5
@@ -241,10 +245,6 @@ class DerivedComponent(Component):
         """ Reassign the Data object that this DerivedComponent operates on """
         self._data = data
 
-    @property
-    def hidden(self):
-        return self._link.hidden
-
     @property
     def data(self):
         """ Return the numerical data as a numpy array """
@@ -300,6 +300,12 @@ class CoordinateComponent(Component):
             if view is None:
                 view = Ellipsis
 
+            # If the view is a tuple or list of arrays, we should actually just
+            # convert these straight to world coordinates since the indices
+            # of the pixel coordinates are the pixel coordinates themselves.
+            if isinstance(view, (tuple, list)) and isinstance(view[0], np.ndarray):
+                return np.array(self._data.coords.pixel2world(*view[::-1])[::-1])[self.axis]
+
             # For 1D arrays, slice can be given as a single slice but we need
             # to wrap it in a list to make the following code work correctly,
             # as it is then consistent with higher-dimensional cases.
@@ -435,9 +441,6 @@ class CategoricalComponent(Component):
         if self._categorical_data.ndim > 1:
             raise ValueError("Categorical Data must be 1-dimensional")
 
-        # Disable changing of categories
-        self._categorical_data.setflags(write=False)
-
         self._categories = categories
         self._jitter_method = jitter
         self._is_jittered = False
@@ -681,7 +684,6 @@ class CategoricalComponent(Component):
 
                 return CategoricalMultiRangeSubsetState(selection, att, other_att)
 
-
     def to_series(self, **kwargs):
         """ Convert into a pandas.Series object.
 
@@ -693,3 +695,31 @@ class CategoricalComponent(Component):
 
         return pd.Series(self._categorical_data.ravel(),
                          dtype=np.object, **kwargs)
+
+
+class DateTimeComponent(Component):
+    """
+    A component representing a date/time.
+
+    Parameters
+    ----------
+    data : `~numpy.ndarray`
+        The data to store, with `~numpy.datetime64` dtype
+    """
+
+    def __init__(self, data, units=None):
+
+        self.units = units
+
+        if not isinstance(data, np.ndarray) or data.dtype.kind != 'M':
+            raise TypeError("DateTimeComponent should be initialized with a datetim64 Numpy array")
+
+        self._data = data
+
+    @property
+    def numeric(self):
+        return True
+
+    @property
+    def datetime(self):
+        return True
