@@ -10,7 +10,9 @@ from glue.external import six
 from glue.core.message import (DataUpdateMessage, DataRemoveComponentMessage,
                                DataAddComponentMessage, NumericalDataChangedMessage,
                                SubsetCreateMessage, ComponentsChangedMessage,
-                               ComponentReplacedMessage)
+                               ComponentReplacedMessage, DataReorderComponentMessage,
+                               ExternallyDerivableComponentsChangedMessage,
+                               PixelAlignedDataChangedMessage)
 from glue.core.decorators import clear_cache
 from glue.core.util import split_component_view
 from glue.core.hub import Hub
@@ -22,7 +24,6 @@ from glue.core.visual import VisualAttributes
 from glue.core.coordinates import Coordinates
 from glue.core.contracts import contract
 from glue.config import settings
-from glue.utils import view_shape
 
 
 # Note: leave all the following imports for component and component_id since
@@ -77,6 +78,8 @@ class Data(object):
 
         # Components
         self._components = OrderedDict()
+        self._externally_derivable_components = OrderedDict()
+        self._pixel_aligned_data = OrderedDict()
         self._pixel_component_ids = ComponentIDList()
         self._world_component_ids = ComponentIDList()
 
@@ -93,7 +96,7 @@ class Data(object):
 
         self.style = VisualAttributes(parent=self)
 
-        self._coordinate_links = None
+        self._coordinate_links = []
 
         self.data = self
         self.label = label
@@ -157,7 +160,11 @@ class Data(object):
         else:
             if len(self._components) == 0:
                 return True
-            return component.shape == self.shape
+            else:
+                if all(comp.shape == () for comp in self._components.values()):
+                    return True
+                else:
+                    return component.shape == self.shape
 
     @contract(cid=ComponentID, returns=np.dtype)
     def dtype(self, cid):
@@ -175,14 +182,29 @@ class Data(object):
         :param component_id: the component to remove
         :type component_id: :class:`~glue.core.component_id.ComponentID`
         """
+        # TODO: avoid too many messages when removing a component triggers
+        # the removal of derived components.
         if component_id in self._components:
             self._components.pop(component_id)
+            self._removed_derived_that_depend_on(component_id)
             if self.hub:
                 msg = DataRemoveComponentMessage(self, component_id)
                 self.hub.broadcast(msg)
                 msg = ComponentsChangedMessage(self)
                 self.hub.broadcast(msg)
 
+    def _removed_derived_that_depend_on(self, component_id):
+        """
+        Remove internal derived components that can no longer be derived.
+        """
+        remove = []
+        for cid in self.derived_components:
+            comp = self.get_component(cid)
+            if component_id in comp.link.get_from_ids():
+                remove.append(cid)
+        for cid in remove:
+            self.remove_component(cid)
+
     @contract(other='isinstance(Data)',
               cid='cid_like',
               cid_other='cid_like')
@@ -355,7 +377,7 @@ class Data(object):
         other._key_joins[self] = (cid_other, cid)
 
     @contract(component='component_like', label='cid_like')
-    def add_component(self, component, label, hidden=False):
+    def add_component(self, component, label):
         """ Add a new component to this data set.
 
         :param component: object to add. Can be a Component,
@@ -381,12 +403,14 @@ class Data(object):
         """
 
         if isinstance(component, ComponentLink):
-            component = DerivedComponent(self, component)
+            return self.add_component_link(component, label=label)
 
         if not isinstance(component, Component):
             component = Component.autotyped(component)
 
         if isinstance(component, DerivedComponent):
+            if len(self._components) == 0:
+                raise TypeError("Cannot add a derived component as a first component")
             component.set_parent(self)
 
         if not(self._check_can_add(component)):
@@ -399,19 +423,23 @@ class Data(object):
             if component_id.parent is None:
                 component_id.parent = self
         else:
-            component_id = ComponentID(label, hidden=hidden, parent=self)
+            component_id = ComponentID(label, parent=self)
+
+        if len(self._components) == 0:
+            # TODO: make sure the following doesn't raise a componentsraised message
+            self._create_pixel_and_world_components(ndim=component.ndim)
+
+        # In some cases, such as when loading a session, we actually disable the
+        # auto-creation of pixel and world coordinates, so the first component
+        # may be a coordinate component with no shape. Therefore we only set the
+        # shape once a component has a valid shape rather than strictly on the
+        # first component.
+        if self._shape == () and component.shape != ():
+            self._shape = component.shape
 
         is_present = component_id in self._components
         self._components[component_id] = component
 
-        first_component = len(self._components) == 1
-        if first_component:
-            if isinstance(component, DerivedComponent):
-                raise TypeError("Cannot add a derived component as "
-                                "first component")
-            self._shape = component.shape
-            self._create_pixel_and_world_components()
-
         if self.hub and not is_present:
             msg = DataAddComponentMessage(self, component_id)
             self.hub.broadcast(msg)
@@ -420,19 +448,82 @@ class Data(object):
 
         return component_id
 
+    def _set_externally_derivable_components(self, derivable_components):
+        """
+        Externally deriable components are components identified by component
+        IDs from other datasets.
+
+        This method is meant for internal use only and is called by the link
+        manager. The ``derivable_components`` argument should be set to a
+        dictionary where the keys are the derivable component IDs, and the
+        values are DerivedComponent instances which can be used to get the
+        data.
+        """
+        if len(self._externally_derivable_components) == 0 and len(derivable_components) == 0:
+            return
+        self._externally_derivable_components = derivable_components
+        if self.hub:
+            msg = ExternallyDerivableComponentsChangedMessage(self)
+            self.hub.broadcast(msg)
+
+    def _set_pixel_aligned_data(self, pixel_aligned_data):
+        """
+        Pixel-aligned data are datasets that contain pixel component IDs
+        that are equivalent (identically, not transformed) with all pixel
+        component IDs in the present dataset.
+
+        Note that the other datasets may have more but not fewer dimensions, so
+        this information may not be symmetric between datasets with differing
+        numbers of dimensions.
+        """
+
+        # First check if anything has changed, as if not then we should just
+        # leave things as-is and avoid emitting a message.
+        if len(self._pixel_aligned_data) == len(pixel_aligned_data):
+            for data in self._pixel_aligned_data:
+                if data not in pixel_aligned_data or pixel_aligned_data[data] != self._pixel_aligned_data[data]:
+                    break
+            else:
+                return
+
+        self._pixel_aligned_data = pixel_aligned_data
+        if self.hub:
+            msg = PixelAlignedDataChangedMessage(self)
+            self.hub.broadcast(msg)
+
+    @property
+    def pixel_aligned_data(self):
+        """
+        Information about other datasets in the same data collection that have
+        matching or a subset of pixel component IDs.
+
+        This is returned as a dictionary where each key is a dataset with
+        matching pixel component IDs, and the value is the order in which the
+        pixel component IDs of the other dataset can be found in the current
+        one.
+        """
+        return self._pixel_aligned_data
+
     @contract(link=ComponentLink,
               label='cid_like|None',
               returns=DerivedComponent)
     def add_component_link(self, link, label=None):
-        """ Shortcut method for generating a new :class:`~glue.core.component.DerivedComponent`
-        from a ComponentLink object, and adding it to a data set.
+        """
+        Shortcut method for generating a new
+        :class:`~glue.core.component.DerivedComponent` from a ComponentLink
+        object, and adding it to a data set.
 
-        :param link: :class:`~glue.core.component_link.ComponentLink`
-        :param label: The ComponentID or label to attach to.
-        :type label: :class:`~glue.core.component_id.ComponentID` or str
+        Parameters
+        ----------
+        link : :class:`~glue.core.component_link.ComponentLink`
+            The link to use to generate a new component
+        label : :class:`~glue.core.component_id.ComponentID` or str
+            The ComponentID or label to attach to.
 
-        :returns:
-            The :class:`~glue.core.component.DerivedComponent` that was added
+        Returns
+        -------
+        component : :class:`~glue.core.component.DerivedComponent`
+            The component that was added
         """
         if label is not None:
             if not isinstance(label, ComponentID):
@@ -443,24 +534,58 @@ class Data(object):
             raise TypeError("Cannot add component_link: "
                             "has no 'to' ComponentID")
 
+        for cid in link.get_from_ids():
+            if cid not in self.components:
+                raise ValueError("Can only add internal links with add_component_link "
+                                 "- use DataCollection.add_link to add inter-data links")
+
         dc = DerivedComponent(self, link)
         to_ = link.get_to_id()
-        self.add_component(dc, to_)
+        self.add_component(dc, label=to_)
         return dc
 
-    def _create_pixel_and_world_components(self):
-        for i in range(self.ndim):
+    def _create_pixel_and_world_components(self, ndim):
+
+        for i in range(ndim):
             comp = CoordinateComponent(self, i)
-            label = pixel_label(i, self.ndim)
-            cid = PixelComponentID(i, "Pixel Axis %s" % label, hidden=True, parent=self)
+            label = pixel_label(i, ndim)
+            cid = PixelComponentID(i, "Pixel Axis %s" % label, parent=self)
             self.add_component(comp, cid)
             self._pixel_component_ids.append(cid)
+
         if self.coords:
-            for i in range(self.ndim):
+            for i in range(ndim):
                 comp = CoordinateComponent(self, i, world=True)
                 label = self.coords.axis_label(i)
-                cid = self.add_component(comp, label, hidden=True)
+                cid = self.add_component(comp, label)
                 self._world_component_ids.append(cid)
+            self._set_up_coordinate_component_links(ndim)
+
+    def _set_up_coordinate_component_links(self, ndim):
+
+        def make_toworld_func(i):
+            def pix2world(*args):
+                return self.coords.pixel2world_single_axis(*args[::-1], axis=ndim - 1 - i)
+            return pix2world
+
+        def make_topixel_func(i):
+            def world2pix(*args):
+                return self.coords.world2pixel_single_axis(*args[::-1], axis=ndim - 1 - i)
+            return world2pix
+
+        result = []
+        for i in range(ndim):
+            link = CoordinateComponentLink(self._pixel_component_ids,
+                                           self._world_component_ids[i],
+                                           self.coords, i)
+            result.append(link)
+            link = CoordinateComponentLink(self._world_component_ids,
+                                           self._pixel_component_ids[i],
+                                           self.coords, i, pixel2world=False)
+            result.append(link)
+
+        self._coordinate_links = result
+        return result
 
     @property
     def components(self):
@@ -470,14 +595,18 @@ class Data(object):
         """
         return list(self._components.keys())
 
+    @property
+    def externally_derivable_components(self):
+        return list(self._externally_derivable_components.keys())
+
     @property
     def visible_components(self):
-        """ :class:`ComponentIDs <glue.core.component_id.ComponentID>` for all non-hidden components.
+        """All :class:`ComponentIDs <glue.core.component_id.ComponentID>` in the Data that aren't coordinate.
 
         :rtype: list
         """
         return [cid for cid, comp in self._components.items()
-                if not cid.hidden and not comp.hidden and cid.parent is self]
+                if not isinstance(comp, CoordinateComponent) and cid.parent is self]
 
     @property
     def coordinate_components(self):
@@ -488,6 +617,11 @@ class Data(object):
         return [c for c in self.component_ids() if
                 isinstance(self._components[c], CoordinateComponent)]
 
+    @property
+    def main_components(self):
+        return [c for c in self.component_ids() if
+                not isinstance(self._components[c], (DerivedComponent, CoordinateComponent))]
+
     @property
     def primary_components(self):
         """The ComponentIDs not associated with a :class:`~glue.core.component.DerivedComponent`
@@ -535,7 +669,7 @@ class Data(object):
             one takes precedence.
         """
 
-        for cid_set in (self.primary_components, self.derived_components):
+        for cid_set in (self.primary_components, self.derived_components, self.coordinate_components, list(self._externally_derivable_components)):
 
             result = []
             for cid in cid_set:
@@ -553,45 +687,27 @@ class Data(object):
         return None
 
     @property
-    def coordinate_links(self):
-        """A list of the ComponentLinks that connect pixel and
-        world. If no coordinate transformation object is present,
-        return an empty list.
+    def links(self):
         """
-        if self._coordinate_links:
-            return self._coordinate_links
+        A list of all the links internal to the dataset.
+        """
+        return self.coordinate_links + self.derived_links
 
-        if not self.coords:
-            return []
+    @property
+    def coordinate_links(self):
+        """
+        A list of the ComponentLinks that connect pixel and world. If no
+        coordinate transformation object is present, return an empty list.
+        """
+        return self._coordinate_links
 
-        if self.ndim != len(self._pixel_component_ids) or \
-                self.ndim != len(self._world_component_ids):
-                # haven't populated pixel, world coordinates yet
-            return []
-
-        def make_toworld_func(i):
-            def pix2world(*args):
-                return self.coords.pixel2world_single_axis(*args[::-1], axis=self.ndim - 1 - i)
-            return pix2world
-
-        def make_topixel_func(i):
-            def world2pix(*args):
-                return self.coords.world2pixel_single_axis(*args[::-1], axis=self.ndim - 1 - i)
-            return world2pix
-
-        result = []
-        for i in range(self.ndim):
-            link = CoordinateComponentLink(self._pixel_component_ids,
-                                           self._world_component_ids[i],
-                                           self.coords, i)
-            result.append(link)
-            link = CoordinateComponentLink(self._world_component_ids,
-                                           self._pixel_component_ids[i],
-                                           self.coords, i, pixel2world=False)
-            result.append(link)
-
-        self._coordinate_links = result
-        return result
+    @property
+    def derived_links(self):
+        """
+        A list of the links present inside all of the DerivedComponent objects
+        in this dataset.
+        """
+        return [self.get_component(cid).link for cid in self.derived_components]
 
     @contract(axis=int, returns=ComponentID)
     def get_pixel_component_id(self, axis):
@@ -695,7 +811,7 @@ class Data(object):
         Send a :class:`~glue.core.message.DataUpdateMessage` to the hub
 
         :param attribute: Name of an attribute that has changed (or None)
-        :type attribute: string
+        :type attribute: str
         """
         if not self.hub:
             return
@@ -746,8 +862,6 @@ class Data(object):
             pass
 
         if changed and self.hub is not None:
-            # promote hidden status
-            new._hidden = new.hidden and old.hidden
 
             # remove old component and broadcast the change
             # see #508 for discussion of this
@@ -758,24 +872,18 @@ class Data(object):
         s = "Data Set: %s\n" % self.label
         s += "Number of dimensions: %i\n" % self.ndim
         s += "Shape: %s\n" % ' x '.join([str(x) for x in self.shape])
-        for hidden in [False, True]:
-            if hidden:
-                s += "Hidden "
-            else:
-                s += "Main "
-            s += "components:\n"
-            if hidden:
-                components = [c for c in self.components if c not in self.visible_components]
-            else:
-                components = [c for c in self.visible_components]
-            for i, cid in enumerate(components):
-                if cid.hidden != hidden:
-                    continue
-                comp = self.get_component(cid)
-                if comp.units is None or comp.units == '':
-                    s += " %i) %s\n" % (i, cid)
-                else:
-                    s += " %i) %s [%s]\n" % (i, cid, comp.units)
+        categories = [('Main', self.main_components),
+                      ('Derived', self.derived_components),
+                      ('Coordinate', self.coordinate_components)]
+        for category, components in categories:
+            if len(components) > 0:
+                s += category + " components:\n"
+                for cid in components:
+                    comp = self.get_component(cid)
+                    if comp.units is None or comp.units == '':
+                        s += " - {0}\n".format(cid)
+                    else:
+                        s += " - {0} [{1}]\n".format(cid, comp.units)
         return s[:-1]
 
     def __repr__(self):
@@ -812,13 +920,13 @@ class Data(object):
         if isinstance(key, ComponentLink):
             return key.compute(self, view)
 
-        try:
+        if key in self._components:
             comp = self._components[key]
-        except KeyError:
+        elif key in self._externally_derivable_components:
+            comp = self._externally_derivable_components[key]
+        else:
             raise IncompatibleAttribute(key)
 
-        shp = view_shape(self.shape, view)
-
         if view is not None:
             result = comp[view]
         else:
@@ -827,8 +935,6 @@ class Data(object):
             else:
                 result = comp.data
 
-        assert result.shape == shp, \
-            "Component view returned bad shape: %s %s" % (result.shape, shp)
         return result
 
     def __setitem__(self, key, value):
@@ -849,9 +955,11 @@ class Data(object):
         if isinstance(component_id, six.string_types):
             component_id = self.id[component_id]
 
-        try:
+        if component_id in self._components:
             return self._components[component_id]
-        except KeyError:
+        elif component_id in self._externally_derivable_components:
+            return self._externally_derivable_components[component_id]
+        else:
             raise IncompatibleAttribute(component_id)
 
     def to_dataframe(self, index=None):
@@ -867,6 +975,39 @@ class Data(object):
         order = [comp.label for comp in self.components]
         return df[order]
 
+    def reorder_components(self, component_ids):
+        """
+        Reorder the components using a list of component IDs. The new set
+        of component IDs has to match the existing set (though order may differ).
+        """
+
+        # We need to be careful because component IDs overload == so we can't
+        # use the normal ways to test whether the component IDs are the same
+        # as self.components - instead we need to explicitly use id
+
+        if len(component_ids) != len(self.components):
+            raise ValueError("Number of component in component_ids does not "
+                             "match existing number of components")
+
+        if set(id(c) for c in self.components) != set(id(c) for c in component_ids):
+            raise ValueError("specified component_ids should match existing components")
+
+        existing = self.components
+        for idx in range(len(component_ids)):
+            if component_ids[idx] is not existing[idx]:
+                break
+        else:
+            # If we get here then the suggested order is the same as the existing one
+            return
+
+        # PY3: once we drop support for Python 2 we could sort in-place using
+        # the move_to_end method on OrderedDict
+        self._components = OrderedDict((key, self._components[key]) for key in component_ids)
+
+        if self.hub:
+            msg = DataReorderComponentMessage(self, list(self._components))
+            self.hub.broadcast(msg)
+
     @contract(mapping="dict(inst($Component, $ComponentID):array_like)")
     def update_components(self, mapping):
         """
