@@ -7,20 +7,25 @@ import operator as op
 
 import pytest
 import numpy as np
-from numpy.testing import assert_equal
+from numpy.testing import assert_equal, assert_allclose
 
 from mock import MagicMock
 
-from glue.tests.helpers import requires_astropy
-
+from glue.tests.helpers import requires_astropy, requires_scipy, SCIPY_INSTALLED
+from ..exceptions import IncompatibleAttribute
 from .. import DataCollection, ComponentLink
 from ..data import Data, Component
-from ..roi import CategoricalROI, RectangularROI
+from ..roi import CategoricalROI, RectangularROI, Projected3dROI, CircularROI
 from ..message import SubsetDeleteMessage
 from ..registry import Registry
+from ..link_helpers import LinkSame
 from ..subset import (Subset, SubsetState,
                       ElementSubsetState, RoiSubsetState, RangeSubsetState,
-                      CategoricalROISubsetState, InequalitySubsetState, CategorySubsetState, MaskSubsetState, CategoricalROISubsetState2D, CategoricalMultiRangeSubsetState)
+                      CategoricalROISubsetState, InequalitySubsetState,
+                      CategorySubsetState, MaskSubsetState,
+                      CategoricalROISubsetState2D, RoiSubsetState3d,
+                      CategoricalMultiRangeSubsetState, FloodFillSubsetState,
+                      SliceSubsetState)
 from ..subset import AndState
 from ..subset import InvertState
 from ..subset import OrState
@@ -181,6 +186,7 @@ class TestSubset(object):
         with pytest.raises(TypeError):
             s.subset_state = 5
 
+
 target_states = ((op.and_, AndState),
                  (op.or_, OrState),
                  (op.xor, XorState))
@@ -451,7 +457,6 @@ class TestSubsetViews(object):
 
 
 def roifac(comp, cid):
-    from ..roi import RectangularROI
 
     result = RoiSubsetState()
     result.xatt = cid
@@ -462,6 +467,16 @@ def roifac(comp, cid):
     return result
 
 
+def roifac3d(comp, cid):
+
+    roi_2d = RectangularROI(0.5, 0.5, 1.5, 1.5)
+    roi_3d = Projected3dROI(roi_2d=roi_2d, projection_matrix=np.arange(16).reshape((4, 4)))
+
+    result = RoiSubsetState3d(xatt=cid, yatt=cid, zatt=cid, roi=roi_3d)
+
+    return result
+
+
 def rangefac(comp, cid):
     return RangeSubsetState(.5, 2.5, att=cid)
 
@@ -500,6 +515,18 @@ def basefac(comp, cid):
     return SubsetState()
 
 
+def maskfac(comp, cid):
+    return MaskSubsetState([[0, 1], [1, 1]], cid.parent.pixel_component_ids)
+
+
+def floodfac(comp, cid):
+    return FloodFillSubsetState(cid.parent, cid, [0, 0], 1.2)
+
+
+def slicefac(comp, cid):
+    return SliceSubsetState(cid.parent, [slice(None), slice(0, 1)])
+
+
 views = (np.s_[:],
          np.s_[::-1, 0],
          np.s_[0, :],
@@ -508,8 +535,11 @@ views = (np.s_[:],
          np.where(np.array([[True, False], [False, True]])),
          np.zeros((2, 2), dtype=bool),
          )
-facs = [roifac, rangefac, orfac, andfac, xorfac, invertfac,
-        elementfac, inequalityfac, basefac]
+facs = [roifac, roifac3d, rangefac, orfac, andfac, xorfac, invertfac,
+        elementfac, inequalityfac, basefac, maskfac, slicefac]
+
+if SCIPY_INSTALLED:
+    facs.append(floodfac)
 
 
 @pytest.mark.parametrize(('statefac', 'view'), [(f, v) for f in facs
@@ -528,9 +558,7 @@ def test_mask_of_view_is_view_of_mask(statefac, view):
     np.testing.assert_array_equal(v1, v2)
 
     v1 = s[cid, view]
-    m0 = np.zeros_like(c.data, dtype=bool)
-    m0[view] = True
-    v2 = c.data[s.to_mask() & m0]
+    v2 = c.data[view][s.to_mask(view)]
     np.testing.assert_array_equal(v1, v2)
 
 
@@ -712,7 +740,7 @@ class TestCloneSubsetStates():
 
     def test_category_multi_range_subset_state(self):
 
-        ranges = {'a': [(1.0, 1.1), (1.3, 1.6)], 'b':[(1.1, 1.4), (1.7, 1.8)], 'c':[(1.1, 1.2)]}
+        ranges = {'a': [(1.0, 1.1), (1.3, 1.6)], 'b': [(1.1, 1.4), (1.7, 1.8)], 'c': [(1.1, 1.2)]}
 
         subset = self.data.new_subset()
         subset.subset_state = CategoricalMultiRangeSubsetState(ranges, self.data.id['b'], self.data.id['c'])
@@ -810,3 +838,128 @@ class TestCloneSubsetStates():
         data_clone = clone(self.data)
 
         assert_equal(data_clone.subsets[0].to_mask(), [0, 1, 0, 0])
+
+
+@requires_scipy
+def test_floodfill_subset_state():
+
+    data = np.array([[9, 6, 2, 3],
+                     [4, 5, 2, 5],
+                     [2, 4, 1, 0],
+                     [5, 6, 0, -1]])
+
+    expected = np.array([[0, 0, 0, 0],
+                         [1, 1, 0, 0],
+                         [0, 1, 0, 0],
+                         [0, 0, 0, 0]])
+
+    data1 = Data(x=data)
+    data2 = Data(x=4 - data1['x'])
+
+    dc = DataCollection([data1, data2])
+
+    subset_state = FloodFillSubsetState(data1, data1.id['x'], (1, 0), 1.3)
+
+    dc.new_subset_group(label='subset', subset_state=subset_state)
+
+    result = data1.subsets[0].to_mask()
+    assert_equal(result, expected)
+
+    with pytest.raises(IncompatibleAttribute):
+        data2.subsets[0].to_mask()
+
+    # Check that setting up pixel links works
+
+    dc.add_link(LinkSame(data1.pixel_component_ids[0], data2.pixel_component_ids[0]))
+    dc.add_link(LinkSame(data1.pixel_component_ids[1], data2.pixel_component_ids[1]))
+
+    result = data2.subsets[0].to_mask()
+    assert_equal(result, expected)
+
+    dc._link_manager.clear_links()
+
+    dc.add_link(LinkSame(data1.pixel_component_ids[1], data2.pixel_component_ids[0]))
+    dc.add_link(LinkSame(data1.pixel_component_ids[0], data2.pixel_component_ids[1]))
+
+    result = data2.subsets[0].to_mask()
+    assert_equal(result, expected.T)
+
+    # Check that (de)serialization works
+
+    dc_new = clone(dc)
+
+    result = dc_new[0].subsets[0].to_mask()
+    assert_equal(result, expected)
+
+    # Check that changing parameters invalidates the cache
+
+    dc[0].subsets[0].subset_state.threshold = 10
+    result = data1.subsets[0].to_mask()
+    assert_equal(result, 1)
+
+
+def test_projected_3d_clone():
+
+    d = Data(x=[1, 2, 3], y=[2, 3, 4], z=[4, 3, 2])
+    roi_2d = CircularROI(2, 3, 4)
+    projection_matrix = np.random.uniform(-1, 1, 16).reshape((4, 4))
+    roi_3d = Projected3dROI(roi_2d=roi_2d, projection_matrix=projection_matrix)
+
+    subset_state = RoiSubsetState3d(d.id['y'], d.id['x'], d.id['z'], roi_3d)
+
+    subset_state_new = clone(subset_state)
+
+    assert subset_state_new.xatt.label == 'y'
+    assert subset_state_new.yatt.label == 'x'
+    assert subset_state_new.zatt.label == 'z'
+    assert isinstance(subset_state_new.roi, Projected3dROI)
+    assert_allclose(subset_state_new.roi.projection_matrix, projection_matrix)
+
+
+def test_slice_subset_state():
+
+    data1 = Data(x=np.arange(24).reshape((2, 3, 4)))
+
+    slices = [slice(None), slice(1, 3), slice(None, None, 2)]
+    subset_state = SliceSubsetState(data1, slices)
+
+    expected_mask = np.zeros((2, 3, 4))
+    expected_mask[slices] = 1
+    assert_equal(subset_state.to_mask(data1), expected_mask)
+
+    view = [slice(0, 1), slice(None), slice(None)]
+    assert_equal(subset_state.to_mask(data1, view=view), expected_mask[view])
+
+    data2 = Data(x=np.arange(24).reshape((3, 4, 2)))
+    data_collection = DataCollection([data1, data2])
+
+    assert_equal(subset_state.to_mask(data2), 0)
+
+    data_collection.add_link(LinkSame(data1.pixel_component_ids[0], data2.pixel_component_ids[2]))
+
+    assert_equal(subset_state.to_mask(data2), 0)
+
+    data_collection.add_link(LinkSame(data1.pixel_component_ids[1], data2.pixel_component_ids[0]))
+    data_collection.add_link(LinkSame(data1.pixel_component_ids[2], data2.pixel_component_ids[1]))
+
+    assert_equal(subset_state.to_mask(data2), expected_mask.transpose().swapaxes(0, 1))
+
+    view = [slice(None), slice(1, 3), slice(None)]
+    assert_equal(subset_state.to_mask(data2, view=view), expected_mask.transpose().swapaxes(0, 1)[view])
+
+
+def test_slice_subset_state_clone():
+
+    data1 = Data(x=np.arange(24).reshape((2, 3, 4)))
+
+    slices = [slice(None), slice(1, 3), slice(None, None, 2)]
+    subset_state = SliceSubsetState(data1, slices)
+    subset = data1.new_subset()
+    subset.subset_state = subset_state
+
+    expected_mask = np.zeros((2, 3, 4))
+    expected_mask[slices] = 1
+    assert_equal(data1.subsets[0].to_mask(), expected_mask)
+
+    data2 = clone(data1)
+    assert_equal(data2.subsets[0].to_mask(), expected_mask)
