@@ -8,12 +8,12 @@ except ImportError:  # Python 2.7
     from inspect import getargspec as getfullargspec
 
 from qtpy import QtWidgets
-from qtpy import PYSIDE
 from glue import core
 from glue.config import link_function, link_helper
 from glue.utils import nonpartial
-from glue.utils.qt import load_ui, messagebox_on_error, update_combobox
-from glue.utils.qt.widget_properties import CurrentComboTextProperty, CurrentComboDataProperty
+from glue.utils.qt import load_ui, messagebox_on_error
+from glue.external.echo import SelectionCallbackProperty
+from glue.external.echo.qt import connect_combo_selection
 
 __all__ = ['LinkEquation']
 
@@ -125,26 +125,27 @@ class LinkEquation(QtWidgets.QWidget):
        widget = LinkEquation()
     """
 
-    category = CurrentComboTextProperty('_ui.category')
-    function = CurrentComboDataProperty('_ui.function')
+    category = SelectionCallbackProperty()
+    function = SelectionCallbackProperty()
 
     def __init__(self, parent=None):
         super(LinkEquation, self).__init__(parent)
 
         # Set up mapping of function/helper name -> function/helper tuple. For the helpers, we use the 'display' name if available.
         self._argument_widgets = []
-        self.spacer = None
         self._output_widget = ArgumentWidget("")
 
         # pyqt4 can't take self as second argument here
         # for some reason. Manually embed
         self._ui = load_ui('link_equation.ui', None,
                            directory=os.path.dirname(__file__))
+
         l = QtWidgets.QHBoxLayout()
         l.addWidget(self._ui)
         self.setLayout(l)
 
-        self._init_widgets()
+        self._ui.outputs_layout.addWidget(self._output_widget)
+
         self._populate_category_combo()
         self.category = 'General'
         self._populate_function_combo()
@@ -152,7 +153,7 @@ class LinkEquation(QtWidgets.QWidget):
         self._setup_editor()
 
     def set_result_visible(self, state):
-        self._ui.output_canvas.setVisible(state)
+        self._output_widget.setVisible(state)
         self._ui.output_label.setVisible(state)
 
     def is_helper(self):
@@ -163,26 +164,6 @@ class LinkEquation(QtWidgets.QWidget):
         return self.function is not None and \
             type(self.function).__name__ == 'LinkFunction'
 
-    def _init_widgets(self):
-
-        layout = QtWidgets.QVBoxLayout()
-        layout.setSpacing(1)
-        self._ui.input_canvas.setLayout(layout)
-
-        layout = QtWidgets.QVBoxLayout()
-        layout.setContentsMargins(1, 0, 1, 1)
-        self._ui.output_canvas.setLayout(layout)
-
-        layout.addWidget(self._output_widget)
-        spacer = QtWidgets.QSpacerItem(5, 5,
-                                       QtWidgets.QSizePolicy.Minimum,
-                                       QtWidgets.QSizePolicy.Expanding)
-        layout.addItem(spacer)
-
-        font = self._ui.info.font()
-        font.setPointSize(font.pointSize() * 1.4)
-        self._ui.info.setFont(font)
-
     @property
     def signature(self):
         """ Returns the ComponentIDs assigned to the input and output arguments
@@ -245,7 +226,7 @@ class LinkEquation(QtWidgets.QWidget):
     def _setup_editor(self):
         if self.is_function():
             self._setup_editor_function()
-        else:
+        elif self.is_helper():
             self._setup_editor_helper()
 
     def _setup_editor_function(self):
@@ -261,10 +242,6 @@ class LinkEquation(QtWidgets.QWidget):
         for a in args:
             self._add_argument_widget(a)
 
-        self.spacer = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Minimum,
-                                        QtWidgets.QSizePolicy.Expanding)
-        self._ui.input_canvas.layout().addItem(self.spacer)
-
     def _setup_editor_helper(self):
         """Setup the editor for the selected link helper"""
         assert self.is_helper()
@@ -277,39 +254,34 @@ class LinkEquation(QtWidgets.QWidget):
         for a in args:
             self._add_argument_widget(a)
 
-        self.spacer = QtWidgets.QSpacerItem(5, 5, QtWidgets.QSizePolicy.Minimum,
-                                        QtWidgets.QSizePolicy.Expanding)
-        self._ui.input_canvas.layout().addItem(self.spacer)
-
     def _add_argument_widget(self, argument):
         """ Create and add a single argument widget to the input canvas
         :param arguement: The argument name (string)
         """
         widget = ArgumentWidget(argument)
         widget.editor.textChanged.connect(nonpartial(self._update_add_enabled))
-        self._ui.input_canvas.layout().addWidget(widget)
+        self._ui.inputs_layout.addWidget(widget)
         self._argument_widgets.append(widget)
 
     def _clear_input_canvas(self):
         """ Remove all widgets from the input canvas """
-        layout = self._ui.input_canvas.layout()
+        layout = self._ui.inputs_layout
         for a in self._argument_widgets:
             layout.removeWidget(a)
             a.close()
 
-        if not PYSIDE:
-            # PySide crashing here
-            layout.removeItem(self.spacer)
-
         self._argument_widgets = []
 
     def _populate_category_combo(self):
         f = [f for f in link_function.members if len(f.output_labels) == 1]
         categories = sorted(set(l.category for l in f + link_helper.members))
-        update_combobox(self._ui.category, list(zip(categories, categories)))
+        LinkEquation.category.set_choices(self, categories)
+        connect_combo_selection(self, 'category', self._ui.category)
 
     def _populate_function_combo(self):
         """ Add name of functions to function combo box """
         f = [f for f in link_function.members if len(f.output_labels) == 1]
-        functions = ((get_function_name(l[0]), l) for l in f + link_helper.members if l.category == self.category)
-        update_combobox(self._ui.function, functions)
+        functions = [l for l in f + link_helper.members if l.category == self.category]
+        LinkEquation.function.set_choices(self, functions)
+        LinkEquation.function.set_display_func(self, lambda l: get_function_name(l[0]))
+        connect_combo_selection(self, 'function', self._ui.function)
