@@ -44,6 +44,23 @@ class TestImageCommon(BaseTestMatplotlibDataViewer):
     def test_double_add_ignored(self):
         pass
 
+    def test_slice_change_single_draw(self):
+
+        # Regression test for a bug that caused Matplotlib to draw once per
+        # data/subset when changing slices.
+
+        self.viewer.add_data(self.data)
+
+        self.data_collection.new_subset_group(label='a', subset_state=self.data.id['x'] > 1)
+        self.data_collection.new_subset_group(label='b', subset_state=self.data.id['x'] > 2)
+        self.data_collection.new_subset_group(label='c', subset_state=self.data.id['x'] > 3)
+
+        self.init_draw_count()
+
+        assert self.draw_count == 0
+        self.viewer.state.slices = (1, 1, 1)
+        assert self.draw_count == 1
+
 
 class MyCoords(Coordinates):
     def axis_label(self, i):
@@ -90,6 +107,9 @@ class TestImageViewer(object):
 
     def teardown_method(self, method):
         self.viewer.close()
+        self.viewer = None
+        self.application.close()
+        self.application = None
 
     def test_basic(self):
 
@@ -97,8 +117,8 @@ class TestImageViewer(object):
 
         self.viewer.add_data(self.image1)
 
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'World 0:World 1'
-        assert combo_as_string(self.options_widget.ui.combosel_y_att_world) == 'World 0:World 1'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:World 0:World 1'
+        assert combo_as_string(self.options_widget.ui.combosel_y_att_world) == 'Coordinate components:World 0:World 1'
 
         assert self.viewer.axes.get_xlabel() == 'World 1'
         assert self.viewer.state.x_att_world is self.image1.id['World 1']
@@ -125,8 +145,8 @@ class TestImageViewer(object):
 
         self.viewer.add_data(self.image2)
 
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Banana:Apple'
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Banana:Apple'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:Banana:Apple'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:Banana:Apple'
 
         assert self.viewer.axes.get_xlabel() == 'Apple'
         assert self.viewer.state.x_att_world is self.image2.id['Apple']
@@ -252,8 +272,8 @@ class TestImageViewer(object):
 
         self.viewer.add_data(self.hypercube)
 
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'World 0:World 1:World 2:World 3'
-        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'World 0:World 1:World 2:World 3'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:World 0:World 1:World 2:World 3'
+        assert combo_as_string(self.options_widget.ui.combosel_x_att_world) == 'Coordinate components:World 0:World 1:World 2:World 3'
 
         assert self.viewer.axes.get_xlabel() == 'World 3'
         assert self.viewer.state.x_att_world is self.hypercube.id['World 3']
@@ -291,6 +311,21 @@ class TestImageViewer(object):
         self.viewer.add_data(self.image1)
         self.data_collection.new_subset_group(subset_state=self.catalog.id['c'] > 1, label='A')
 
+    def test_invisible_subset(self):
+
+        # Regression test for a bug that caused a subset layer that started
+        # off as invisible to have issues when made visible. We emulate the
+        # initial invisible (but enabled) state by invalidating the cache.
+
+        self.viewer.add_data(self.image1)
+        self.data_collection.new_subset_group(subset_state=self.image1.id['x'] > 1, label='A')
+        self.viewer.layers[1].visible = False
+        self.viewer.layers[1].image_artist.invalidate_cache()
+        self.viewer.layers[1].redraw()
+        assert not np.any(self.viewer.layers[1].image_artist._A.mask)
+        self.viewer.layers[1].visible = True
+        assert not np.any(self.viewer.layers[1].image_artist._A.mask)
+
     def test_apply_roi_single(self):
 
         # Regression test for a bug that caused mode.update to be called
@@ -298,7 +333,7 @@ class TestImageViewer(object):
         # messages regarding subset updates (this occurred when multiple)
         # datasets were present.
 
-        layer_tree = LayerTreeWidget()
+        layer_tree = LayerTreeWidget(session=self.session)
         layer_tree.set_checkable(False)
         layer_tree.setup(self.data_collection)
         layer_tree.bind_selection_to_edit_subset()
@@ -617,7 +652,7 @@ class TestImageViewer(object):
         self.viewer.add_data(self.hypercube)
         self.viewer.state.reference_data = self.hypercube
 
-        assert self.viewer.layers[1].subset_array.shape is None
+        assert self.viewer.layers[1].subset_array.shape == (4, 5)
         assert self.viewer.layers[3].subset_array.shape == (4, 5)
 
     def test_preserve_slice(self):
